@@ -2,13 +2,12 @@ import itertools
 import queue
 import time
 import weakref
-from typing import List, Any, Tuple, Optional, Callable, Union, Match, AnyStr, Generator
-from xmlrpc.client import ResponseError
+from typing import List, Any, Tuple, Optional, Callable, Union, Match, AnyStr, Generator, Sequence
 
 import redis
-from redis.connection import DefaultParser
+from redis import ResponseError
 
-from fakeredis.model import XStream, ZSet, Hash, ExpiringMembersSet
+from fakeredis.model import XStream, ZSet, Hash, ExpiringMembersSet, ClientInfo
 from . import _msgs as msgs
 from ._command_args_parsing import extract_args
 from ._commands import Int, Float, SUPPORTED_COMMANDS, COMMANDS_WITH_SUB, Signature, CommandItem
@@ -24,6 +23,20 @@ from ._helpers import (
 )
 
 
+def _convert_to_resp2(val: Any) -> Any:
+    if isinstance(val, str):
+        return val.encode()
+    if isinstance(val, float):
+        return Float.encode(val, humanfriendly=False)
+    if isinstance(val, dict):
+        result = list(itertools.chain(*val.items()))
+        return [_convert_to_resp2(item) for item in result]
+    if isinstance(val, (list, tuple)):
+        res = [_convert_to_resp2(item) for item in val]
+        return res
+    return val
+
+
 def _extract_command(fields: List[bytes]) -> Tuple[Any, List[Any]]:
     """Extracts the command and command arguments from a list of `bytes` fields.
 
@@ -67,13 +80,14 @@ class BaseFakeSocket:
     }
     _connection_error_class = redis.ConnectionError
 
-    def __init__(self, server: "FakeServer", db: int, *args: Any, **kwargs: Any) -> None:  # type: ignore # noqa: F821
+    def __init__(self, server: "FakeServer", db: int, client_class, *args: Any, **kwargs: Any) -> None:  # type: ignore # noqa: F821
         super(BaseFakeSocket, self).__init__(*args, **kwargs)
         from fakeredis import FakeServer
 
         self._server: FakeServer = server
         self._db_num = db
         self._db = server.dbs[self._db_num]
+        self._client_class = client_class
         self.responses: Optional[queue.Queue[bytes]] = queue.Queue()
         # Prevents parser from processing commands. Not used in this module,
         # but set by aioredis module to prevent new commands being processed
@@ -86,8 +100,13 @@ class BaseFakeSocket:
         self._in_transaction: bool
         self._pubsub: int
         self._transaction_failed: bool
-        self._current_user: bytes = b"default"
-        self._client_info: bytes = kwargs.pop("client_info", b"")
+        info = kwargs.pop("client_info", dict(user="default"))
+        self._client_info = ClientInfo(**info)
+        self._server.sockets.append(self)
+
+    @property
+    def current_user(self) -> bytes:
+        return self._client_info.user
 
     @property
     def version(self) -> Tuple[int, ...]:
@@ -143,6 +162,7 @@ class BaseFakeSocket:
         # redis.Connection.__del__, which the garbage collection could call
         # at any time, and hence we can't safely take the server lock.
         # We rely on list.append being atomic.
+        self._server.sockets.remove(self)
         self._server.closed_sockets.append(weakref.ref(self))
         self._server = None  # type: ignore
         self._db = None
@@ -190,7 +210,8 @@ class BaseFakeSocket:
         cmd, cmd_arguments = _extract_command(fields)
         try:
             func, sig = self._name_to_func(cmd)
-            self._server.acl.validate_command(self._current_user, self._client_info, fields)  # ACL check
+            # ACL check
+            self._server.acl.validate_command(self._client_info.user, self._client_info.as_bytes(), fields)
             with self._server.lock:
                 # Clean out old connections
                 while True:
@@ -241,7 +262,12 @@ class BaseFakeSocket:
             else:
                 args, command_items = ret
                 result = func(*args)  # type: ignore
-                assert valid_response_type(result)
+                if self._client_info.protocol_version == 2 and msgs.FLAG_SKIP_CONVERT_TO_RESP2 not in sig.flags:
+                    result = _convert_to_resp2(result)
+                if msgs.FLAG_SKIP_CONVERT_TO_RESP2 not in sig.flags:
+                    assert valid_response_type(result, self._client_info.protocol_version), (
+                        f"Invalid response type for {result}"
+                    )
         except SimpleError as exc:
             result = exc
         for command_item in command_items:
@@ -249,7 +275,14 @@ class BaseFakeSocket:
         return result
 
     def _decode_error(self, error: SimpleError) -> ResponseError:
-        return DefaultParser(socket_read_size=65536).parse_error(error.value)  # type: ignore
+        if self._client_class.__module__.startswith("valkey"):
+            from valkey.connection import DefaultParser
+
+            return DefaultParser(socket_read_size=65536).parse_error(error.value)
+        else:
+            from redis.connection import DefaultParser
+
+            return DefaultParser(socket_read_size=65536).parse_error(error.value)
 
     def _decode_result(self, result: Any) -> Any:
         """Convert SimpleString and SimpleError, recursively"""
@@ -308,7 +341,7 @@ class BaseFakeSocket:
             data = data.encode("ascii")  # type: ignore
         self._parser.send(data)
 
-    def _scan(self, keys, cursor, *args):
+    def _scan(self, keys: Sequence[bytes], cursor: int, *args: bytes) -> List[Union[bytes, List[bytes]]]:
         """This is the basis of most of the ``scan`` methods.
 
         This implementation is KNOWN to be un-performant, as it requires grabbing the full set of keys over which
@@ -344,16 +377,18 @@ class BaseFakeSocket:
         bits_len = (len(keys) - 1).bit_length()
         cursor = bin_reverse(cursor, bits_len)
         if cursor >= len(keys):
-            return [0, []]
+            return [b"0", []]
         result_cursor = cursor + count
         result_data = []
 
         regex = compile_pattern(pattern) if pattern is not None else None
 
         def match_key(key: bytes) -> Union[bool, Match[bytes], None]:
+            if isinstance(key, str):
+                key = key.encode("utf-8")
             return regex.match(key) if regex is not None else True
 
-        def match_type(key) -> bool:
+        def match_type(key: bytes) -> bool:
             return _type is None or casematch(BaseFakeSocket._key_value_type(self._db[key]).value, _type)
 
         if pattern is not None or _type is not None:
