@@ -8,10 +8,11 @@ import math
 import re
 import sys
 import time
-from typing import Iterable, Tuple, Union, Optional, Any, Type, List, Callable, Sequence, Dict, Set, Collection
+from typing import Tuple, Union, Optional, Any, Type, List, Callable, Sequence, Dict, Set, Collection
 
 from . import _msgs as msgs
-from ._helpers import null_terminate, SimpleError, Database, current_time
+from ._helpers import null_terminate, SimpleError, Database
+from ._typing import VersionType, ServerType
 
 MAX_STRING_SIZE = 512 * 1024 * 1024
 SUPPORTED_COMMANDS: Dict[str, "Signature"] = dict()  # Dictionary of supported commands name => Signature
@@ -107,82 +108,6 @@ class CommandItem:
     __nonzero__ = __bool__  # For Python 2
 
 
-class Hash:
-    DECODE_ERROR = msgs.INVALID_HASH_MSG
-    redis_type = b"hash"
-
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
-        super().__init__(*args, **kwargs)
-        self._expirations: Dict[bytes, int] = dict()
-        self._values: Dict[bytes, Any] = dict()
-
-    def _expire_keys(self) -> None:
-        removed = []
-        now = current_time()
-        for k in self._expirations:
-            if self._expirations[k] < now:
-                self._values.pop(k, None)
-                removed.append(k)
-        for k in removed:
-            self._expirations.pop(k, None)
-
-    def set_key_expireat(self, key: bytes, when_ms: int) -> int:
-        now = current_time()
-        if when_ms <= now:
-            self._values.pop(key, None)
-            self._expirations.pop(key, None)
-            return 2
-        self._expirations[key] = when_ms
-        return 1
-
-    def clear_key_expireat(self, key: bytes) -> bool:
-        return self._expirations.pop(key, None) is not None
-
-    def get_key_expireat(self, key: bytes) -> Optional[int]:
-        self._expire_keys()
-        return self._expirations.get(key, None)
-
-    def __getitem__(self, key: bytes) -> Any:
-        self._expire_keys()
-        return self._values.get(key)
-
-    def __contains__(self, key: bytes) -> bool:
-        self._expire_keys()
-        return self._values.__contains__(key)
-
-    def __setitem__(self, key: bytes, value: Any) -> None:
-        self._expirations.pop(key, None)
-        self._values[key] = value
-
-    def __delitem__(self, key: bytes) -> None:
-        self._values.pop(key, None)
-        self._expirations.pop(key, None)
-
-    def __len__(self) -> int:
-        return len(self._values)
-
-    def __iter__(self) -> Iterable[bytes]:
-        return iter(self._values)
-
-    def get(self, key: bytes, default: Any = None) -> Any:
-        return self._values.get(key, default)
-
-    def keys(self) -> Iterable[bytes]:
-        self._expire_keys()
-        return self._values.keys()
-
-    def values(self) -> Iterable[Any]:
-        return [v for k, v in self.items()]
-
-    def items(self) -> Iterable[Tuple[bytes, Any]]:
-        self._expire_keys()
-        return self._values.items()
-
-    def update(self, values: Dict[bytes, Any]) -> None:
-        self._expire_keys()
-        self._values.update(values)
-
-
 class RedisType:
     @classmethod
     def decode(cls, *args, **kwargs):  # type:ignore
@@ -425,7 +350,7 @@ class Signature:
         repeat: Tuple[Type[Union[RedisType, bytes]]] = (),  # type:ignore
         args: Tuple[str] = (),  # type:ignore
         flags: str = "",
-        server_types: Collection[str] = (
+        server_types: Collection[ServerType] = (
             "redis",
             "valkey",
             "dragonfly",
@@ -437,9 +362,9 @@ class Signature:
         self.repeat = repeat
         self.flags = set(flags)
         self.command_args = args
-        self.server_types: Set[str] = set(server_types)
+        self.server_types: Set[ServerType] = set(server_types)
 
-    def check_arity(self, args: Sequence[Any], version: Tuple[int]) -> None:
+    def check_arity(self, args: Sequence[Any], version: VersionType) -> None:
         if len(args) == len(self.fixed):
             return
         delta = len(args) - len(self.fixed)
@@ -451,7 +376,7 @@ class Signature:
             raise SimpleError(msg)
 
     def apply(
-        self, args: Sequence[Any], db: Database, version: Tuple[int]
+        self, args: Sequence[Any], db: Database, version: VersionType
     ) -> Union[Tuple[Any], Tuple[List[Any], List[CommandItem]]]:
         """Returns a tuple, which is either:
         - transformed args and a dict of CommandItems; or
@@ -469,7 +394,7 @@ class Signature:
             if isinstance(type_, Key):
                 if type_.missing_return is not Key.UNSPECIFIED and arg not in db:
                     return (type_.missing_return,)
-            elif type_ != bytes:
+            elif type_ is not bytes:
                 args_list[i] = type_.decode(
                     args_list[i],
                 )
@@ -556,7 +481,7 @@ class Timestamp(Int):
     @classmethod
     def decode(cls, value: bytes, decode_error: Optional[str] = None) -> int:
         if value == b"*":
-            return int(time.time())
+            return int(time.time() * 1000)
         if value == b"-":
             return -1
         if value == b"+":
